﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Threading;
using Ozeki.Media;
using Ozeki.VoIP;

namespace IVR.IVR.Menu
{
    class MenuElement
    {
        protected string greetingMessage;
        protected char returnDtmfCommand;
        protected ICall call;
        protected MediaConnector mediaConnector;
        protected TextToSpeech textToSpeech;
        protected WaveStreamPlayback wavPlayback;
        protected PhoneCallAudioSender phoneCallAudioSender;
        protected Dictionary<char, MenuElement> submenuElements;

        public MenuElement(string greetingMessage, char returnDtmfCommand)
        {
            this.greetingMessage = greetingMessage;
            mediaConnector = new MediaConnector();
            textToSpeech = new TextToSpeech();
            phoneCallAudioSender = new PhoneCallAudioSender();
            this.returnDtmfCommand = returnDtmfCommand;

            submenuElements = new Dictionary<char, MenuElement>();
        }

        public MenuElement(WaveStreamPlayback wavPlayback, char returnDtmfCommand)
        {
            this.wavPlayback = wavPlayback;
            this.returnDtmfCommand = returnDtmfCommand;
            mediaConnector = new MediaConnector();
            phoneCallAudioSender = new PhoneCallAudioSender();

            this.returnDtmfCommand = returnDtmfCommand;
            submenuElements = new Dictionary<char, MenuElement>();
        }

        public virtual void AddSubmenu(MenuElement menuElement, char dtmfCommand)
        {
            if (dtmfCommand == returnDtmfCommand)
            {
                return;
            }
            submenuElements.Add(dtmfCommand, menuElement);
        }

        public event EventHandler Completed;

        public void Execute(ICall call)
        {
            this.call = call;
            Enable();
        }

        protected virtual void DtmfReceived(object sender, VoIPEventArgs<DtmfInfo> e)
        {
            char dtmfChar = DtmfNumToChar(e.Item.Signal.Signal);
            if (dtmfChar == returnDtmfCommand)
            {
                OnCompleted();
                return;
            }

            MenuElement submenu;
            submenuElements.TryGetValue(dtmfChar, out submenu);

            if (submenu != null)
            {
                Disable();
                submenu.Completed += OnChildCompleted;
                submenu.Execute(call);
            }
        }

        protected virtual void Enable()
        {
            BackgroundWorker playInvoker = new BackgroundWorker();
            
            call.DtmfReceived += DtmfReceived;

            if (textToSpeech != null)
            {

                mediaConnector.Connect(textToSpeech, phoneCallAudioSender);
                phoneCallAudioSender.AttachToCall(call);
                playInvoker.DoWork += delegate
                                            {
                                                Thread.Sleep(500);
                                                textToSpeech.AddAndStartText(greetingMessage);
                                            };
            }
            else if (wavPlayback != null)
            {
                mediaConnector.Connect(wavPlayback, phoneCallAudioSender);
                phoneCallAudioSender.AttachToCall(call);
                playInvoker.DoWork += delegate
                                            {
                                                Thread.Sleep(500);
                                                wavPlayback.Start();
                                            };
            }
           
            playInvoker.RunWorkerAsync();
        }

        protected virtual void Disable()
        {
            call.DtmfReceived -= DtmfReceived;
            if (textToSpeech != null)
            {
                if (textToSpeech.IsStreaming)
                {
                    textToSpeech.Stop();
                }
            }
            else if(wavPlayback != null)
            {
                if (wavPlayback.IsStreaming)
                {
                    wavPlayback.Stop();
                }
            }
            phoneCallAudioSender.Detach();
            mediaConnector.Dispose();
        }

        protected virtual void OnCompleted()
        {
            Disable();

            var handler = Completed;
            if (handler != null)
                handler(this, EventArgs.Empty);
        }

        protected virtual void OnChildCompleted(object sender, EventArgs e)
        {
            Enable();
            ((MenuElement)sender).Completed -= OnChildCompleted;
        }

        protected static char DtmfNumToChar(int dtmf)
        {
            if (dtmf > 9)
            {
                return dtmf == 10 ? '*' : '#';
            }
            return dtmf.ToString(CultureInfo.InvariantCulture)[0];
        }

        protected static int DtmfCharToNum(char dtmf)
        {
            if (dtmf == '*')
            {
                return 10;
            }

            if (dtmf == '#')
            {
                return 11;
            }

            return int.Parse(dtmf.ToString(CultureInfo.InvariantCulture));
        }
    }
}
